// #############################################################################
// #                         --- LED Scroller ---                              #
// #############################################################################
// # main.c - Main program                                                     #
// #############################################################################
// #              Version: 1.0 - Compiler: AVR-GCC 4.5.3 (Linux)               #
// #  (c) 2011 by Malte Pöggel - www.MALTEPOEGGEL.de - malte@maltepoeggel.de   #
// #############################################################################

// PINS:
// PD2 = Row 1
// PD3 = Row 2
// PD4 = Row 3
// PD5 = Row 4
// PD6 = Row 5
// PB0 = Col 1
// PB1 = Col 2
// PB2 = Col 3
// PB3 = Col 4
// PB4 = Col 5
// PB5 = Col 6
// PB6 = Col 7

 #include <avr/io.h>
 #include <avr/interrupt.h>
 #include <util/delay.h>
 #include "main.h"
 #include "font.h"

 prog_uchar message_a[] = {
  ' ', 'M', 'a', 'n', ' ', 'i', 's', 't', ' ', 'g', 'e', 's', 'c', 'h', 'o', 'c', 'k', 't', ' ', 'u', 'n', 'd', ' ', 'a', 'u', 'c', 'h', ' ', 'v', 'e', 'r', 'w', 'i', 'r', 'r', 't', ',', ' ', 'w', 'e', 'n', 'n', ' ', 'm', 'a', 'n', ' ', 'e', 'r', 's', 't', ' ', 'm', 'a', 'l', ' ', '3', '0', ' ', 'w', 'i', 'r', 'd', '.',
  ' ', 'H', 'a', 'p', 'p', 'y', ' ', 'B', 'i', 'r', 't', 'h', 'd', 'a', 'y', ',', 'H', 'o', 'l', 'g', 'e', 'r', '!', ' ', ':', '-', ')', '\0'
  };

 prog_uchar message_b[] = {
  ' ', '3', '0', ' ', 'w', 'i', 'r', 's', 't', ' ', 'D', 'u', ' ', 'k', 'e', 'i', 'n', 'e', ' ', 'F', 'r', 'a', 'g', 'e', ',', ' ', 'd', 'a', 's', ' ', 'b', 'l', 'e', 'i', 'b','t', ' ', 's', 'o', ' ', 'n', 'u', 'n', ' ', 'v', 'i', 'e', 'l', 'e', ' ', 'T', 'a', 'g', 'e', '.',
  ' ', 'H', 'a', 'p', 'p', 'y', ' ', 'B', 'i', 'r', 't', 'h', 'd', 'a', 'y', ',', 'H', 'o', 'l', 'g', 'e', 'r', '!', ' ', ':', '-', ')', '\0'
  };

 // Main program
 int main( void )
  {
   uint8_t x, y;
   
   // Set Output
   DDRD = 0xFF;
   DDRB = 0xFF;

   // Setup prescaler (8MHz / 64) and overflow interrupt
   TCCR0B = (1<<CS00) | (1<<CS01);       
   TIMSK |= (1<<TOIE0);

   // Init globals and enable interrupts
   matrixcounter = 0;
   sei();

   // Endless loop 8-)
   for(;;)
    {  
     // Message A
     x = 0; y = 0;
     while(putchars(x, y++, (prog_uchar*)&message_a))
      { 
       if(y>5) 
        {
         y=0;
         x++;
        }
       _delay_ms(150);
      }
     _delay_ms(2000);
     // Message B
     x = 0; y = 0;
     while(putchars(x, y++, (prog_uchar*)&message_b))
      { 
       if(y>5) 
        {
         y=0;
         x++;
        }
       _delay_ms(150);
      }
     _delay_ms(2000); 
    }

  }


 // For horizontal character scrolling
 uint8_t putchars( uint8_t pos, uint8_t scroll, prog_uchar* text )
  {
   int i;
   uint8_t chardata;
   for(i=0; i<5; i++)
    {
     if((5-scroll)>i)
      {
       chardata = pgm_read_byte(&Font5x7[(5*(uint8_t)pgm_read_byte(text+pos))+i+scroll]);
      } else {
       if((5-scroll)!=i)
        {
         chardata = pgm_read_byte(&Font5x7[(5*(uint8_t)pgm_read_byte(text+pos+1))+(i-(6-scroll))]); 
        } else {
         chardata = 0x00;
        }     
      }     
     matrixdata[i] = chardata;
    }
   if(pgm_read_byte(text+pos)=='\0') return 0;
   return 1;
  }
  

 // Timer ISR (LED multiplexing)
 ISR( TIMER0_OVF_vect ) 
  {
   PORTB = 0x00;
   PORTD = (1<<(matrixcounter+2));
   PORTB = matrixdata[matrixcounter++];
   if(matrixcounter>=5) matrixcounter = 0;
  }
